# =============================================================================
#　skBLEsub.py
# skBLEtool.py用のコンソール画面を表示させる為のスクリプトです。
# =============================================================================
# VERSION  DATE        BY                    CHANGE/COMMENT
# -----------------------------------------------------------------------------
# 1.00     2021-05-07  きむ茶工房(きむしげ)  Create
# 1.20     2021-07-17  きむ茶工房(きむしげ)  HEXデータでもwrite出来る様にした
# -----------------------------------------------------------------------------
# IDE : PyCharm(COMMUNITY 2019.2) Python 3.7
# wxPython : 4.0.6
# PDS(Public Domain Software)
# =============================================================================
import asyncio
import wx
from bleak import BleakClient

class win_mon():
    """ テキストコントロールをコンソール出力画面として作成します """
    DEVICE_ADDRESS = ''
    CHARACTERISTIC_UUID = []
    CHARACTERISTIC_ATTR = []  # 選択されたキャラクタリスティックの属性(ハンドル・種類)を保存

    def __init__(self, mon_win):
        # テキストコントロールの作成
        self.mon_textctrl = wx.TextCtrl(mon_win, wx.ID_ANY,
                                        style=wx.TE_MULTILINE | wx.HSCROLL | wx.VSCROLL | wx.TE_PROCESS_ENTER)
        self.mon_textctrl.SetPosition((5, 220))
        self.mon_textctrl.Bind(wx.EVT_TEXT_ENTER, self.mon_textctrl_enter)

    def obj(self):
        """ コンソール出力画面のインスタンス変数を返す """
        return self.mon_textctrl

    def write(self,text):
        """ 標準出力がコンソール出力画面に出力を切り替えられる様にする """
        self.mon_textctrl.AppendText(text)

    def cls(self):
        """ コンソール出力画面をクリアする """
        self.mon_textctrl.Clear()

    def mon_textctrl_enter(self, event):
        """ コンソール画面のテキスト入力時にEnterキーを押された時のイベントハンドラー """
        async def run_write(data, kind, handle):
            _client = BleakClient(win_mon.DEVICE_ADDRESS)
            await _client.connect(timeout=20.0)  # 接続
            if _client.is_connected:
                # 0xが付いていたらHEXで送る
                _dt = b''
                if data[0] == '0' and data[1] == 'x':
                    import struct
                    print('HEX!')
                    _l = len(data[2:])
                    for i  in range(0,_l,2):
                        _p = i + 2
                        _dt += struct.pack('B', int(data[_p:_p+2], 16))
                else:
                    _dt = data.encode()
                # 書き込み操作を実行します(応答は無し)。
                if kind == 'char':
                    await _client.write_gatt_char(self.CHARACTERISTIC_UUID[0], _dt)
                    print("\nchar sent:", _dt)
                elif (kind == '┗desc') or (kind == 'desc'):
                    await _client.write_gatt_descriptor(int(handle), _dt)
                    print("\ndesc sent:", _dt)
                else:
                    print('キャラクタリスティック/ディスクリプタの書き込み属性では無い様です。')
                await _client.disconnect()  # 接続切断
            else:
                print('\nタイマーアウトみたいな')

        if len(self.CHARACTERISTIC_UUID) == 0:
            print('\nキャラクタリスティックUUIDが選択されていません。', end='')
            return
        _obj = event.GetEventObject()
        _p = _obj.GetInsertionPoint()              # カーソルの位置を座標で得る
        _boo, _pos, _line = _obj.PositionToXY(_p)  # 座標からpos,lineに変換する
        data = _obj.GetLineText(_line)             # 入力データを得る
        _s = self.CHARACTERISTIC_ATTR[0].split(':')
        print('\nデバイスへ write中', end='')
        loop = asyncio.get_event_loop()
        loop.run_until_complete(run_write(data, _s[0], _s[1]))
# end of class win_mon

