# =============================================================================
#　skDICT.py
# 辞書データを作成してファイルとして読み書きできます。
# =============================================================================
# VERSION  DATE        BY                    CHANGE/COMMENT
# -----------------------------------------------------------------------------
# 1.00     2021-05-07  きむ茶工房(きむしげ)    Create
# -----------------------------------------------------------------------------
# IDE : PyCharm(COMMUNITY 2019.2) Python 3.7
# wxPython : 4.0.6
# PDS(Public Domain Software)
# =============================================================================
import pickle

class DICT():
    OVERWRITE   = 1  # 上書きする
    NOOVERWRITE = 0  # 上書きしない

    def __init__(self, flag=OVERWRITE):
        self.DICT_DT = {}
        self.f = None
        self.flag = flag
        self.access_flag = False

    def open(self, fname):
        """ 辞書ファイルのオープン """
        self.file = fname + ".pkl"
        try:
            self.f = open(self.file, "rb")
        except OSError:
            # なければ作成する
            self.f = open(self.file, "wb")
        else:
            try:
                self.DICT_DT = pickle.load(self.f)  # 辞書ファイルを読み込む
            except EOFError:
                pass

    def close(self):
        """ 辞書ファイルのクローズ """
        if self.access_flag:
            # 書き込まれていたなら処理する
            self.f.close()                          # 取り合えず一旦クローズ
            self.f = open(self.file, "wb")          # 書き込みモードで開き直す
            pickle.dump(self.DICT_DT, self.f)       # 辞書ファイルを書き込む
        self.f.close()

    def put(self, key, data):
        """ 指定のキー値とデータを書き込む """
        if self.flag == self.NOOVERWRITE:
            # 上書きしない場合はキー値が無ければ書き込む
            try:
                _bdt = self.DICT_DT[key]
            except KeyError:
                self.access_flag = True
                self.DICT_DT[key] = data
            else:
                # キー値が存在するので例外エラーを発生させる
                raise KeyError('The key exists')
        else:
            # 上書きand新規書き込み
            self.access_flag = True
            self.DICT_DT[key] = data

    def get(self, key):
        """ 指定キー値のデータを読み出す """
        try:
            _bdt = self.DICT_DT[key]
        except KeyError:
            return None  # そのキー値は無い
        return _bdt

    def delete(self, key):
        """ 指定のキー値を削除する """
        try:
            del self.DICT_DT[key]
        except KeyError:
            return None  # そのキー値は無い
        else:
            self.access_flag = True

    def keys(self):
        """ キーを全て読み込む """
        _s = []
        for _bkey in self.DICT_DT.keys():
            _s.append(_bkey)
        return _s

    def values(self):
        """ 値を全て読み込む """
        _s = []
        for _bdata in self.DICT_DT.values():
            _s.append(_bdata)
        return _s
# end of class DICT

